import { App } from 'vue';
import { InjectionKey } from 'vue';

export declare type CheckboxInput = InputBase & {
    type: 'checkbox';
    value: boolean;
};

export declare type ColorInput = InputBase & {
    type: 'color';
    value: string;
};

export declare function createDynamicForms(options?: DynamicFormsOptions): DynamicFormsPlugin;

export declare type CustomInput = InputBase & {
    type: 'custom-field';
    value: boolean | string | number;
};

export declare interface DynamicFormsOptions {
    autoValidate?: boolean;
    form?: FormOptions;
}

export declare interface DynamicFormsPlugin {
    options?: DynamicFormsOptions;
    install(app: App): void;
}

export declare const dynamicFormsSymbol: InjectionKey<DynamicFormsPlugin>;

export declare const email: (control: FormControl<InputType>) => ValidationErrors | null;

export declare type EmailInput = InputBase & {
    type: 'email';
    value: string;
};

export declare type FormControl<T extends InputType> = T & {
    valid: boolean;
    invalid: boolean;
    dirty: boolean;
    touched: boolean;
    errors: ValidationErrors | null;
};

export declare interface FormOptions {
    customClass?: string;
    method?: string;
    netlify?: boolean;
    netlifyHoneypot?: string;
}

export declare interface FormValidation {
    validator: ValidatorFn;
    text: string;
}

export declare interface InputBase {
    name: string;
    label?: string;
    disabled?: boolean;
    customClass?: string;
    placeholder?: string;
    validations?: FormValidation[];
}

export declare type InputType = TextInput | NumberInput | SelectInput | TextAreaInput | CheckboxInput | RadioInput | EmailInput | PasswordInput | RadioInput | ColorInput | UrlInput | CustomInput;

export declare type InputTypeKeys = 'text' | 'email' | 'url' | 'number' | 'password' | 'textarea' | 'select' | 'checkbox' | 'radio' | 'color' | 'custom-field';

export declare const isEmptyInputValue: (value: string | number | boolean) => boolean;

export declare const max: (max: number) => (control: FormControl<InputType>) => ValidationErrors | null;

export declare const maxLength: (maxLength: number) => (control: FormControl<InputType>) => ValidationErrors | null;

export declare const min: (min: number) => (control: FormControl<InputType>) => ValidationErrors | null;

export declare const minLength: (minLength: number) => (control: FormControl<InputType>) => ValidationErrors | null;

export declare type NumberInput = InputBase & {
    type: 'number';
    value: number;
    min?: number;
    max?: number;
    step?: number;
};

export declare type PasswordInput = InputBase & {
    type: 'password';
    value: string;
};

export declare const pattern: (pattern: string) => ValidationErrors | null;

export declare type RadioInput = InputBase & {
    type: 'radio';
    value: string;
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
};

export declare const required: (control: FormControl<InputType>) => ValidationErrors | null;

export declare type SelectInput<T = boolean | string> = InputBase & {
    type: 'select';
    value: T;
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
};

export declare type TextAreaInput = InputBase & {
    type: 'textarea';
    value: string;
    cols?: number;
    rows?: number;
};

export declare type TextInput = InputBase & {
    type: 'text';
    value: string;
};

export declare const url: (control: FormControl<InputType>) => ValidationErrors | null;

export declare type UrlInput = InputBase & {
    type: 'url';
    value: string;
};

export declare function useDynamicForms(): DynamicFormsPlugin;

export declare type ValidationErrors = {
    [key: string]: any;
};

declare interface ValidatorFn {
    (control: FormControl<InputType> | undefined): ValidationErrors | null;
}

export { }
