export declare type InputTypeKeys = 'text' | 'email' | 'url' | 'number' | 'password' | 'textarea' | 'select' | 'checkbox' | 'radio' | 'color' | 'custom-field';
export declare type InputType = TextInput | NumberInput | SelectInput | TextAreaInput | CheckboxInput | RadioInput | EmailInput | PasswordInput | RadioInput | ColorInput | UrlInput | CustomInput;
export declare type ValidationErrors = {
    [key: string]: any;
};
interface ValidatorFn {
    (control: FormControl<InputType> | undefined): ValidationErrors | null;
}
export interface FormValidation {
    validator: ValidatorFn;
    text: string;
}
export interface InputBase {
    name: string;
    label?: string;
    disabled?: boolean;
    customClass?: string;
    placeholder?: string;
    validations?: FormValidation[];
}
export declare type TextInput = InputBase & {
    type: 'text';
    value: string;
};
export declare type NumberInput = InputBase & {
    type: 'number';
    value: number;
    min?: number;
    max?: number;
    step?: number;
};
export declare type SelectInput<T = boolean | string> = InputBase & {
    type: 'select';
    value: T;
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
};
export declare type TextAreaInput = InputBase & {
    type: 'textarea';
    value: string;
    cols?: number;
    rows?: number;
};
export declare type CheckboxInput = InputBase & {
    type: 'checkbox';
    value: boolean;
};
export declare type CustomInput = InputBase & {
    type: 'custom-field';
    value: boolean | string | number;
};
export declare type EmailInput = InputBase & {
    type: 'email';
    value: string;
};
export declare type PasswordInput = InputBase & {
    type: 'password';
    value: string;
};
export declare type ColorInput = InputBase & {
    type: 'color';
    value: string;
};
export declare type UrlInput = InputBase & {
    type: 'url';
    value: string;
};
export declare type RadioInput = InputBase & {
    type: 'radio';
    value: string;
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
};
export declare type FormControl<T extends InputType> = T & {
    valid: boolean;
    invalid: boolean;
    dirty: boolean;
    touched: boolean;
    errors: ValidationErrors | null;
};
export interface FormOptions {
    customClass?: string;
    method?: string;
    netlify?: boolean;
    netlifyHoneypot?: string;
}
export {};
//# sourceMappingURL=models.d.ts.map